]> de.git.xonotic.org Git - xonotic/darkplaces.git/blobdiff - cl_demo.c
fixed very annoying '5 second timeout' bug in Host_ShutdownServer (the local client...
[xonotic/darkplaces.git] / cl_demo.c
index 78f14fe21a9740f790a91e1a7792450808af23ac..85efe355531da7633771881f5178b64fa08bcdb7 100644 (file)
--- a/cl_demo.c
+++ b/cl_demo.c
@@ -54,7 +54,7 @@ void CL_NextDemo (void)
                cls.demonum = 0;
                if (!cls.demos[cls.demonum][0])
                {
-                       Con_Print("No demos listed with startdemos\n");
+                       Con_Print("No demos listed with startdemos\n");
                        cls.demonum = -1;
                        return;
                }
@@ -157,6 +157,13 @@ void CL_ReadDemoMessage(void)
                                // count against the final report
                                if (host_framecount == cls.td_startframe)
                                        cls.td_starttime = realtime;
+                               if (host_framecount > cls.td_startframe + 2)
+                               {
+                                       cls.td_minframetime = min(cls.td_minframetime, host_realframetime);
+                                       cls.td_maxframetime = max(cls.td_maxframetime, host_realframetime);
+                               }
+                               else
+                                       cls.td_minframetime = cls.td_maxframetime = host_realframetime;
                        }
                        else if (cl.time <= cl.mtime[0])
                        {
@@ -205,7 +212,7 @@ void CL_Stop_f (void)
 
        if (!cls.demorecording)
        {
-               Con_Print("Not recording a demo.\n");
+               Con_Print("Not recording a demo.\n");
                return;
        }
 
@@ -218,7 +225,7 @@ void CL_Stop_f (void)
        FS_Close (cls.demofile);
        cls.demofile = NULL;
        cls.demorecording = false;
-       Con_Print("Completed demo\n");
+       Con_Print("Completed demo\n");
 }
 
 /*
@@ -239,19 +246,19 @@ void CL_Record_f (void)
        c = Cmd_Argc();
        if (c != 2 && c != 3 && c != 4)
        {
-               Con_Print("record <demoname> [<map> [cd track]]\n");
+               Con_Print("record <demoname> [<map> [cd track]]\n");
                return;
        }
 
        if (strstr(Cmd_Argv(1), ".."))
        {
-               Con_Print("Relative pathnames are not allowed.\n");
+               Con_Print("Relative pathnames are not allowed.\n");
                return;
        }
 
        if (c == 2 && cls.state == ca_connected)
        {
-               Con_Printf("Can not record - already connected to server\nClient demo recording must be started before connecting\n");
+               Con_Print("Can not record - already connected to server\nClient demo recording must be started before connecting\n");
                return;
        }
 
@@ -259,7 +266,7 @@ void CL_Record_f (void)
        if (c == 4)
        {
                track = atoi(Cmd_Argv(3));
-               Con_Printf ("Forcing CD track to %i\n", cls.forcetrack);
+               Con_Printf("Forcing CD track to %i\n", cls.forcetrack);
        }
        else
                track = -1;
@@ -273,16 +280,16 @@ void CL_Record_f (void)
                Cmd_ExecuteString ( va("map %s", Cmd_Argv(2)), src_command);
 
        // open the demo file
-       Con_Printf ("recording to %s.\n", name);
+       Con_Printf("recording to %s.\n", name);
        cls.demofile = FS_Open (name, "wb", false);
        if (!cls.demofile)
        {
-               Con_Print("ERROR: couldn't open.\n");
+               Con_Print("ERROR: couldn't open.\n");
                return;
        }
 
        cls.forcetrack = track;
-       FS_Printf (cls.demofile, "%i\n", cls.forcetrack);
+       FS_Printf(cls.demofile, "%i\n", cls.forcetrack);
 
        cls.demorecording = true;
 }
@@ -306,12 +313,13 @@ void CL_PlayDemo_f (void)
 
        if (Cmd_Argc() != 2)
        {
-               Con_Print("play <demoname> : plays a demo\n");
+               Con_Print("play <demoname> : plays a demo\n");
                return;
        }
 
        // disconnect from server
        CL_Disconnect ();
+       Host_ShutdownServer (false);
 
        // update networking ports (this is mainly just needed at startup)
        NetConn_ClientFrame();
@@ -320,17 +328,18 @@ void CL_PlayDemo_f (void)
        strlcpy (name, Cmd_Argv(1), sizeof (name));
        FS_DefaultExtension (name, ".dem", sizeof (name));
 
-       Con_Printf ("Playing demo from %s.\n", name);
+       Con_Printf("Playing demo from %s.\n", name);
        cls.demofile = FS_Open (name, "rb", false);
        if (!cls.demofile)
        {
-               Con_Print("ERROR: couldn't open.\n");
+               Con_Print("ERROR: couldn't open.\n");
                cls.demonum = -1;               // stop demo loop
                return;
        }
 
        SCR_BeginLoadingPlaque ();
 
+       strlcpy(cls.demoname, name, sizeof(cls.demoname));
        cls.demoplayback = true;
        cls.state = ca_connected;
        cls.forcetrack = 0;
@@ -353,18 +362,23 @@ CL_FinishTimeDemo
 */
 void CL_FinishTimeDemo (void)
 {
-       int             frames;
-       double  time; // LordHavoc: changed timedemo accuracy to double
+       int frames;
+       double time; // LordHavoc: changed timedemo accuracy to double
+       double fpsmin, fpsavg, fpsmax; // report min/avg/max fps
 
        cls.timedemo = false;
 
 // the first frame didn't count
        frames = (host_framecount - cls.td_startframe) - 1;
        time = realtime - cls.td_starttime;
-       if (!time)
-               time = 1;
-       // LordHavoc: timedemo now prints out 7 digits of fraction
-       Con_Printf ("%i frames %5.7f seconds %5.7f fps\n", frames, time, frames/time);
+       fpsmin = cls.td_maxframetime > 0 ? 1.0 / cls.td_maxframetime : 0;
+       fpsavg = time > 0 ? frames / time : 0;
+       fpsmax = cls.td_minframetime > 0 ? 1.0 / cls.td_minframetime : 0;
+       // LordHavoc: timedemo now prints out 7 digits of fraction, and min/avg/max
+       Con_Printf("%i frames %5.7f seconds %5.7f fps\nmin/avg/max: %5.7f/%5.7f/%5.7f\n", frames, time, fpsavg, fpsmin, fpsavg, fpsmax);
+       Con_LogPrintf("benchmark.log", "date %s | enginedate %s | demo %s | commandline %s | result %i frames %5.7f seconds %5.7f fps min/avg/max: %5.7f/%5.7f/%5.7f\n", Sys_TimeString("%Y-%m-%d %H:%M:%S"), buildstring, cls.demoname, cmdline.string, frames, time, fpsavg, fpsmin, fpsavg, fpsmax);
+       if (COM_CheckParm("-benchmark"))
+               Host_Quit_f();
 }
 
 /*
@@ -381,7 +395,7 @@ void CL_TimeDemo_f (void)
 
        if (Cmd_Argc() != 2)
        {
-               Con_Print("timedemo <demoname> : gets demo speeds\n");
+               Con_Print("timedemo <demoname> : gets demo speeds\n");
                return;
        }